"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CannerPATAuthenticator = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@vulcan-sql/core");
const serve_1 = require("@vulcan-sql/serve");
const lodash_1 = require("lodash");
const axios_1 = require("axios");
const config_1 = require("../config");
let CannerPATAuthenticator = class CannerPATAuthenticator extends serve_1.BaseAuthenticator {
    constructor() {
        super(...arguments);
        this.options = {};
    }
    onActivate() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.options = this.getOptions();
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getTokenInfo(ctx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            throw new core_1.InternalError(`canner-pat does not support token generate.`);
        });
    }
    authCredential(context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if ((0, lodash_1.isEmpty)(this.options) || !this.options.host)
                throw new core_1.ConfigurationError('please provide correct connection information to Canner Enterprise, including "host".');
            const authorize = context.request.headers['authorization'];
            // validate request auth token
            const token = authorize.trim().split(' ')[1];
            try {
                return yield this.validate(token);
            }
            catch (err) {
                return {
                    status: serve_1.AuthStatus.FAIL,
                    type: this.getExtensionId(),
                    message: err.message,
                };
            }
        });
    }
    validate(token) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const res = yield this.fetchCannerUser(token);
            const cannerUser = (_a = res.data.data) === null || _a === void 0 ? void 0 : _a.userMe;
            const { username } = cannerUser, restAttrs = tslib_1.__rest(cannerUser, ["username"]);
            return {
                status: serve_1.AuthStatus.SUCCESS,
                type: this.getExtensionId(),
                user: {
                    name: username,
                    attr: restAttrs,
                },
            };
        });
    }
    fetchCannerUser(token) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const graphqlUrl = this.getCannerUrl('/graphql');
            try {
                return yield axios_1.default.post(graphqlUrl, {
                    operationName: 'UserMe',
                    variables: {},
                    query: 'query UserMe{userMe {id accountRole attributes createdAt email groups {id name} lastName firstName username}}',
                }, {
                    headers: {
                        Authorization: `Token ${token}`,
                    },
                });
            }
            catch (error) {
                const message = error.response
                    ? `response status: ${error.response.status}, response data: ${JSON.stringify(error.response.data)}`
                    : `remote server does not response. request ${error.toJSON()}}`;
                throw new core_1.InternalError(`Failed to fetch user info from canner server: ${message}`);
            }
        });
    }
    getCannerUrl(path = '/') {
        const { host, port, ssl = false } = this.options;
        // the web endpoint is different between connecting from internal or external
        if (config_1.default.isOnKubernetes) {
            // the vulcan-sql service and web service is in the same cluster and will not pass the nginx proxy
            return `${process.env['WEB_ENDPOINT']}${path}`;
        }
        else {
            const protocol = ssl ? 'https' : 'http';
            // canner nginx proxy will redirect all request to web service when path starts with /web
            const pathPrefix = '/web';
            const endpoint = `${protocol}://${host}${port ? `:${port}` : ''}`;
            return `${endpoint}${pathPrefix}${path}`;
        }
    }
};
CannerPATAuthenticator = tslib_1.__decorate([
    (0, core_1.VulcanExtensionId)('canner-pat')
], CannerPATAuthenticator);
exports.CannerPATAuthenticator = CannerPATAuthenticator;
//# sourceMappingURL=pat.js.map